/*
 * File   : $Source: /usr/local/cvs/opencms/src/org/opencms/workplace/threads/CmsReportTestThread.java,v $
 * Date   : $Date: 2006/03/27 14:52:27 $
 * Version: $Revision: 1.8 $
 *
 * This library is part of OpenCms -
 * the Open Source Content Mananagement System
 *
 * Copyright (c) 2005 Alkacon Software GmbH (http://www.alkacon.com)
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * For further information about Alkacon Software GmbH, please see the
 * company website: http://www.alkacon.com
 *
 * For further information about OpenCms, please see the
 * project website: http://www.opencms.org
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package org.opencms.workplace.threads;

import org.opencms.file.CmsObject;
import org.opencms.report.A_CmsReportThread;
import org.opencms.report.I_CmsReport;

/**
 * Does a full static export of all system resources in the current site.<p>
 * 
 * @author  Alexander Kandzior 
 * 
 * @version $Revision: 1.8 $ 
 * 
 * @since 6.0.0 
 */
public class CmsReportTestThread extends A_CmsReportThread {

    private int m_count;

    /**
     * Creates a static export Thread.<p>
     * 
     * @param cms the current cms context
     * @param count the count for the test output
     */
    public CmsReportTestThread(CmsObject cms, int count) {

        super(cms, Messages.get().getBundle().key(Messages.GUI_REPORT_TEST_THREAD_NAME_0));
        m_count = count;
        initHtmlReport(cms.getRequestContext().getLocale());
        start();
    }

    /**
     * @see org.opencms.report.A_CmsReportThread#getReportUpdate()
     */
    public String getReportUpdate() {

        return getReport().getReportUpdate();
    }

    /**
     * @see java.lang.Runnable#run()
     */
    public void run() {

        I_CmsReport report = getReport();
        report.println(Messages.get().container(Messages.RPT_TEST_REPORT_BEGIN_0), I_CmsReport.FORMAT_HEADLINE);
        for (int i = 0; i < m_count; i++) {
            report.print(org.opencms.report.Messages.get().container(
                org.opencms.report.Messages.RPT_SUCCESSION_2,
                String.valueOf(i),
                String.valueOf(m_count)));
            report.println(Messages.get().container(Messages.RPT_TEST_REPORT_OUTPUT_0));
            try {
                sleep(250);
            } catch (InterruptedException e) {
                // just continue
            }
        }
        getReport().println(Messages.get().container(Messages.RPT_TEST_REPORT_END_0), I_CmsReport.FORMAT_HEADLINE);
    }
}